#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dlaqps_(integer *m, integer *n, integer *offset, integer *nb, integer *kb, doublereal *a, integer *lda, integer *jpvt, doublereal *tau, doublereal *vn1, doublereal *vn2, doublereal *auxv, doublereal *f, integer *ldf);

#ifdef __cplusplus
}
#endif