#ifdef __cplusplus
extern "C" { 
#endif  

#include "f2c.h" 

int dtrexc_(char *compq, integer *n, doublereal *t, integer *ldt, doublereal *q, integer *ldq, integer *ifst, integer *ilst, doublereal *work, integer *info);

#ifdef __cplusplus
}
#endif